/* OneBitLoader
 * 1 bit boot loader for PIC16F88
 *    made by T.Satoh
 *    Ver0.01 in 2006/01/09
 *    Ver0.02 shorter code by T.Satoh in 2006/01/22
 *    Ver0.03 adapted to RS232C(USART) in 2008/11/15
 */

//#pragma chip PIC16F88
//#include <16F88.h>
#include <hexcodes.h>

#include "serial.h"

#pragma unlockISR	// not generate interrupt code
#pragma resetVector 4

#ifdef SW_SERIAL
#define TOP_ADRS	0x0f7b	// AP start address
#define	PIC_ID	'8'
#else
#define TOP_ADRS	0x0fa5	// AP start address
#define	PIC_ID	'9'
#endif

#define TOP_SPACE	4
#define PROM_ERASE_SIZE	32	// EEPROM erase size
#pragma cdata[ 0 ] = 0x0158a,__CALL(TOP_ADRS+TOP_SPACE)	// PA0 = 1


// define config bit assign
#define conf_CP		(1<<13)		// bit 13
#define conf_CCPMX	(1<<12)		// bit 12
#define conf_DEBUG	(1<<11)		// 1:in-Circuit Dubugger disable
#define conf_WRT(n)	(( n & 0b11)<<9)
#define conf_CPD	(1<<8)		// data EE 1:protect off
#define conf_LVP	(1<<7)
#define conf_BOREN	(1<<6)		// 0:BOR disable
#define conf_MCLRE	(1<<5)
#define conf_PWRTEN_	(1<<3)
#define conf_WDTEN	(1<<2)		// 0:WDT disable

#define conf_FOSC_EXTRC		0b10011
#define conf_FOSC_EXTRC_RA5	0b10010
#define conf_FOSC_INTRC		0b10001
#define conf_FOSC_INTRC_RA5	0b10000
#define conf_FOSC_ECIO		0b00011
#define conf_FOSC_HS		0b00010
#define conf_FOSC_XT		0b00001
#define conf_FOSC_LP		0b00000

#pragma config = conf_CP | conf_DEBUG | conf_WRT( 3 ) | conf_CPD | conf_FOSC_INTRC_RA5

#define	ACK		'o'
#define BUF_LEN	0x40		// buf area length(prom erase blk size x 2)
bank0 uns8 Buf[ BUF_LEN ];	// data buffer
// shrBank uns16 FlashWrAdr;	// Flash mem write pointer
// shrBank uns16 FlashRdAdr;	// Flash mem read pointer
shrBank uns8  LoopCnt;		// data counter
shrBank uns8  DataCount;	// data counter
shrBank uns8  ChkSum;		// data check sum

/* siodriver work area */
shrBank uns8 SioData;		// receive/send data
shrBank uns8 SioBitCnt;		// bit counter
shrBank uns8 SioWaitCounter;	// wait timer counter
shrBank uns16 SioLoopCnt;	// loop counter
//shrBank uns8 Flag;
bit ErrFlg @ STATUS.6;		// RP1 0:error 1:no error


/* prototype declare */
#ifdef SW_SERIAL
page1 void SioPutchar( uns8 );
#endif
page1 uns8 SioGetchar( void );
page1 void FlashCmd( void );
page1 void smain( void );


// this function is not executed, but call smain,
// threfor callpath reachs smain and compiler keep
// local variables area in called functions
void main( void )
{
	smain();
}

#pragma updateBank 0


void dummymain( void )	// dummy
{
	for(;;) {
		RP0 = 0;
		RP1 = 1;
		W = SioGetchar();
		if( RP1 == 1 ) {	// if no error
#ifdef SW_SERIAL
			SioPutchar( W );
#else
//		RP0 = 1;
//		RP1 = 0;
//		while( TRMT == 0 );	// wait if TSR is full
		RP1 = 0;
		TXREG = W;
#endif
		}
	}
}


#pragma origin TOP_ADRS
void dummy( void )
{
	dummymain();
}

#pragma origin TOP_ADRS + TOP_SPACE



page1 void smain( void )
{
	RP0 = 1;
//	RP1 = 0;
#ifdef SW_SERIAL
//	OSCCON = 0b01000000;	// 1MHz
	OSCCON = 0b01010000;	// 2MHz(9600bps)
//	OSCCON = 0b01100000;	// 4MHz
//	RP0 = 1;
//	RP1 = 0;
	ANSEL = 0b00000000;		// 0:dig, 1:ana AN1:active
#else
//	RP0 = 1;
//	RP1 = 0;
//	SPBRG = 3;				// 38.4k(1MHz)
//	SPBRG = 6;				// 9600bps(1MHz)
	SPBRG = 12;				// 9600bps(2MHz)
//	RP0 = 1;
//	RP1 = 0;
//	OSCCON = 0b01000000;	// 1MHz
	OSCCON = 0b01010000;	// 2MHz(9600bps)
//	OSCCON = 0b01100000;	// 4MHz
//	RP0 = 1;
//	RP1 = 0;
	TXSTA = 0b00100100;		// Tx:8bit,Async,HighBaudRate \
#endif
	RP0 = 0;
	RP1 = 1;
	SioGetchar();			// insted of timer wait(->RP0:0,RP1:0)

#ifndef SW_SERIAL
//	RP0 = 0;
//	RP1 = 0;
	RCSTA = 0b10010000;		// Rx:8bit,ContinuasReceive
//	W = RCREG;				// clear receive buffer
#endif


#ifdef SW_SERIAL
//	RP0 = 0;
//	RP1 = 0;
	SioPutchar( PIC_ID );
#else
	W = PIC_ID;
	TXREG = W;
#endif

	for(;;) {
		ChkSum = 0;
		RP1 = 1;	// clear errflg
		W = SioGetchar();
//		RP0 = 0;
//		RP1 = 1;
		EEADRH = W;		// if error occured then write TMR1H
		W = SioGetchar();
//		RP0 = 0;
//		RP1 = 1;
		EEADR = W;		// if error occured then wirte PIR1
		W = PROM_ERASE_SIZE * 2;	// set receive data counter
		LoopCnt = W;
		DataCount = W;
		FSR = &Buf[ 0 ];
		do {
			INDF = SioGetchar();
			FSR++;
		} while( --LoopCnt );
		SioGetchar();		// receive check sum
#if 0	// to be inline asm against compiler bug
		if( Zero_ == 0 || RP1 == 0 ) {	// if check sum error
			RP1 = 0;
			#asm	// compiler bug. assume asm part is one statement
				DW __GOTO( TOP_ADRS )
			#endasm
		}
#else
		#asm
			BTFSS 0x03,RP1
			GOTO GO_AP
			BTFSC	0x03,Zero_
			GOTO CONT
			BCF	0x03,RP1
GO_AP:		DW __GOTO( TOP_ADRS )
CONT:
		#endasm

#endif

		/* write flash memory */
//		RP1 = 1;
		RP0 = 1;
		FREE = 1;	// set block erase
		FlashCmd();	// block erase
		FSR = &Buf[ 0 ];
		FREE = 0;	// reset block erase
		do {
			RP0 = 0;
//			RP1 = 1;
			EEDATA = INDF;
			FSR++;
			EEDATH = INDF;
			FSR++;
			RP0 = 1;
//			RP1 = 1;
			FlashCmd();
			RP0 = 0;
//			RP1 = 1;
			EEADR++;
			DataCount--;
		} while( --DataCount );
		RP0 = 1;
//		RP1 = 1;
		WREN = 0;
		RP0 = 0;
//		RP1 = 1;

#ifdef SW_SERIAL
//		RP0 = 0;
//		RP1 = 0;
		SioPutchar( ACK );	// send OK
#else
//		RP0 = 0;
		RP1 = 0;
		W = ACK;
		TXREG = W;
#endif
	}
}


/* erase 32 word in Flash Programmemory */
void FlashCmd( void )
{
//	RP0 = 1;
//	RP1 = 1;
	EEPGD = 1;	// select Flash Program
	WREN = 1;
	EECON2 = 0x55;
	EECON2 = 0xaa;
	WR = 1;
	#asm
NOP_RET equ 0x0ff0
;NOP_RET equ SioGetchar - 2	;cann't use this statement^^;
;NOP_RET equ $				;cann't use this statement^^;
	#endasm
	nop2();
}


#pragma updateBank 1

#ifdef SW_SERIAL

#include "siodriver.c"

#else

#pragma updateBank 0

// RP0 <- 0
// RP1 <- 0:error, 1:no error
// RP1 -> 0:error, 1:no error
// W   -> receive data

uns8 SioGetchar( void )
{
#if 1
	if( RP1 == 0 ) {	// if error flg on
		return( W );
	}
	SioLoopCnt.low8 = 0;
	do {	// this loop is 1byte then to be as long as possible
		STATUS = 0;
		#asm
//			DW __CALL( SioGetchar - 2 )
			DW __CALL( NOP_RET )
		#endasm
		if( RCIF == 1 ) {	// if receive data exists
			W = RCREG;
			ChkSum += W;
			RP1 = 1;	// set no error
			return( W );
		}
	} while( --SioLoopCnt.low8 );
//	RP1 = 0;	// set error flag

#else
	STATUS = 0;
	if( ErrFlg ) {	// if error occured
		return( W );
	} else {
		SioLoopCnt.high8 = 13;
		do {
			do {		// 3clk loop
//				RP0 = 0;
//				RP1 = 0;
				if( RCIF == 1 ) {	// if receive data exists
					W = RCREG;
					ChkSum += W;
					return( W );
				}
			} while( --SioLoopCnt.low8 );
		} while( --SioLoopCnt.high8 );
		ErrFlg = 1;	// set error flag
	}
#endif
	return( W );
}

#pragma updateBank 1

#endif
